/* This file is really a waste, but sunos lacks prototypes... */

char *getenv();
char *index();
int _flsbuf();
int cfree();
/* extern int free(); */
int close();
int fclose();
void rewind();
int fgetc();
int fprintf();
int fputs();
int fscanf();
int ftime();
int getpid();
int printf();
int puts();
int read();
int scanf();
int sscanf();
int ungetc();
int unlink();
int write();
long atol();
long RANDOM();
long strtol();
void bcopy();
void perror();
void srandom();
void system();
char *memset();
char *_crypt();
int rmdir();
void fseek (FILE *,long,int);
int strftime();
int execl();
void fflush();
int getdtablesize();
void bzero();
int select();
int dup();
int vprintf();
int vsprintf();
int socket();
int setsockopt();
int bind();
int listen();
int accept();
char *strdup_local(char *txt);
int socket();
int setsockopt();
int bind();
int connect();
int listen();
int accept();
int gettimeofday();
void exit();
char *malloc();
