#include <sr-util.h>
#include <sr-logger.h>
#ifndef VMS
#include <malloc.h>
#else
#include <stdlib.h>
#include <sr-general.h>
#endif
#include <errno.h>

char *smalloc ( unsigned int size )
{
   char		*m	= (char *) calloc ( 1, size );

   if ( !m )
   {
      LOG ( facUtil, llevExceptions, "smalloc() failed: %d", errno );
      exit ( 1 );
   }

   return m;
}
