/*
 *   This file was automatically generated by version 1.7 of cextract.
 *   Manual editing not recommended.
 *
 *   Created: Tue Jan 25 16:21:06 1994
 */
#ifndef __CEXTRACT__
#if __STDC__

extern Marcrecord * ReadMarcData ( octetString *os );
extern void ReadMarcData_Data ( Marcdata *marcdata, char *incoming );
extern int ReadMarcData_Directory ( Marcrecord *rec, char *pos );
extern Boolean ReadMarcData_Label ( Record_label *label, char *record, int length );
extern octetString * WriteMarcData ( Marcrecord *rec );
extern octetString WriteMarcData_Label ( Record_label *label );

#else /* __STDC__ */

extern Marcrecord * ReadMarcData (/* octetString *os */);
extern void ReadMarcData_Data (/* Marcdata *marcdata, char *incoming */);
extern int ReadMarcData_Directory (/* Marcrecord *rec, char *pos */);
extern Boolean ReadMarcData_Label (/* Record_label *label, char *record, int length */);
extern octetString * WriteMarcData (/* Marcrecord *rec */);
extern octetString WriteMarcData_Label (/* Record_label *label */);

#endif /* __STDC__ */
#endif /* __CEXTRACT__ */
