#include <malloc.h>
#include <marc.h>
#ifndef __CEXTRACT__
#include <marcproto.h>
#endif

void
FreeMarc(Marcrecord *rec)
{
  FreeMarcEntries(rec->entries);
  free(rec);
}

void
FreeMarcEntries(Marcdata *entry)
{
  if (entry->next)
    FreeMarcEntries(entry->next);
  if (entry->data)
    free(entry->data);
  free(entry);
}
