#include <string.h>
#ifndef VMS
#include <malloc.h>
#else
#include <stdlib.h>
#endif

/* From lib/sr/util.c */

char *strdup (const char *s)
{
   return strcpy ((char *) malloc ((strlen (s)) + 1), s);
}
