
/* 
 * tlv.h  -  do Type Length Value parsing of an BER encodeded ASN.1 stream
 *
 * Copyright (c) 1993 Nordic SR-NET 
 * All Rights reserved 
 * 
 * Written by
 *	Geir Pedersen, Univ of Oslo
 *	Geir.Pedersen@usit.uio.no
 * 
*/


void tlv_open ( int fd );
void tlv_close ( int fd );
char *tlv_parse ( int fd, int *PDUlen );
int tlv_error ( int fd );

