/*
 *   This file was automatically generated by version 1.7 of cextract.
 *   Manual editing not recommended.
 *
 *   Created: Fri Jan 28 14:33:51 1994
 */

extern void delete_all_selections ( void );
extern Boolean delete_selection ( selection *set );
extern selection * find_selection_name ( char *name );
extern selection * find_selection_number ( int number );
extern CloseResponse * handleClose ( CloseRequest *pkt );
extern DeleteResultSetResponse * handleDeleteResultSet ( DeleteResultSetRequest *pkt );
extern OpenResponse * handleOpen ( OpenRequest *pkt );
extern PresentResponse * handlePresent ( PresentRequest *pkt );
extern SearchResponse * handleSearch ( SearchRequest *pkt );
extern void init ( int argc, char **argv, char **env );
extern void parse_args ( int argc, char **argv );

