/*
 *   This file was automatically generated by version 1.7 of cextract.
 *   Manual editing not recommended.
 *
 *   Created: Fri Jan 28 14:33:33 1994
 */
#ifndef __CEXTRACT__
#if __STDC__

extern CV_Status Convert ( char *name_class, char *name_in, char *name_out, void *data_in, void **data_out );
extern Class * FindClass ( char *class_name );
extern Converter * FindConverter ( char *class_name, char *conv_name );
extern void init_converter ( void );
extern void RegisterClassFree ( char *class_name, void (*func)(void *) );
extern void RegisterConverter ( char *class_name, ConverterType type, char *conv_name, void (*func)(void *) );

#else /* __STDC__ */

extern CV_Status Convert (/* char *name_class, char *name_in, char *name_out, void *data_in, void **data_out */);
extern Class * FindClass (/* char *class_name */);
extern Converter * FindConverter (/* char *class_name, char *conv_name */);
extern void init_converter (/* void */);
extern void RegisterClassFree (/* char *class_name, void (*func)(void *) */);
extern void RegisterConverter (/* char *class_name, ConverterType type, char *conv_name, void (*func)(void *) */);

#endif /* __STDC__ */
#endif /* __CEXTRACT__ */
