/*
 * strdup.c,v 1.1 1994/01/28 17:07:08 franktor Exp
 *
 * For now, I put this here, since this is the most common library
 * in the SR distribution, and a strdup() is needed everywhere
 * on certain architectures.
 */

#include <malloc.h>

char *
SR_strdup(const char *str)
{
  char *s = malloc (sizeof(char) * strlen(str) + 1);
  strcpy(s, str);
  return s;
}
