/*
 *   This file was automatically generated by version 1.7 of cextract.
 *   Manual editing not recommended.
 *
 *   Created: Fri Jan 28 14:33:30 1994
 */

extern void add_pointer ( void *pointer );
extern globalData *alloc_globalData ( void );
extern char *append ( char *data, int datalen, char **pkt, int *pkti, int *pktlen );
extern int elemMaxElemSize ( elemDesc *ed );
extern int elemSize ( elemDesc *ed );
extern int elemSizeMax ( elemDesc *ed, int max );
extern char *encode_struct ( char *P, elemDesc *topEd, int *len );
extern globalData *find_globalData ( int fd );
extern void free_globalData ( int fd );
extern void free_out_packet ( globalData *G, qElem *qe );
extern void free_pointers ( void );
extern void free_socket ( int fd );
extern void free_struct ( void *P, elemDesc *topEd );
extern qElem *get_out_packet ( globalData *G );
extern socketStat handle_socket ( int fd, Boolean read, Boolean write, Boolean exception, Boolean *selectForWrite );
extern Boolean initialise_socket ( int fd, elemDesc *sending_struct, elemDesc *recv_struct );
extern void insert ( char *d, int i, char *p );
extern Boolean parse_stream ( globalData *g );
extern Boolean pop ( elemDesc **ed, int *edi, char **p, int *pi, Boolean *b );
extern void push ( elemDesc *ed, int edi, char *p, int pi, Boolean b );
extern void queue_out_packet ( globalData *G, char *p, int len );
extern socketStat read_struct ( int fd, void **p );
extern Boolean send_data ( globalData *G );
extern socketStat send_struct ( int fd, void *p, Boolean *selectForWrite );

