/* automatically generated by pepsy 8.0 #2 (gandalf), do not edit! */

#ifndef	_module_ISO10163__SR__1_defined_
#define	_module_ISO10163__SR__1_defined_

#ifndef	PEPSY_VERSION
#define	PEPSY_VERSION		2
#endif

#include <isode/psap.h>
#include <isode/pepsy.h>
#include <isode/pepsy/UNIV-types.h>


extern modtyp	_ZISO10163_SR_1_mod;
#define _ZAttributeElementISO10163_SR_1	24
#define _ZSearchResponseISO10163_SR_1	27
#define _ZXSR_APDUISO10163_SR_1	1
#define _ZDeleteResultSetRequestISO10163_SR_1	40
#define _ZReferenceIdISO10163_SR_1	44
#define _ZDatabaseRecordISO10163_SR_1	32
#define _ZSearchRequestISO10163_SR_1	15
#define _ZPresentResponseISO10163_SR_1	29
#define _ZProtocolVersionISO10163_SR_1	6
#define _ZAttributesPlusTermISO10163_SR_1	20
#define _ZOperandISO10163_SR_1	19
#define _ZAttributeValueISO10163_SR_1	26
#define _ZAuthenticationISO10163_SR_1	14
#define _ZListStatusesISO10163_SR_1	42
#define _ZRPNStructureISO10163_SR_1	18
#define _ZImplementationVersionISO10163_SR_1	12
#define _ZPreferredRecordSyntaxISO10163_SR_1	39
#define _ZPresentStatusISO10163_SR_1	38
#define _ZAttributeTypeISO10163_SR_1	25
#define _ZImplementationNameISO10163_SR_1	11
#define _ZInitialiseResponseISO10163_SR_1	5
#define _ZInitialiseRequestISO10163_SR_1	4
#define _ZResultSetIdISO10163_SR_1	46
#define _ZRecordsISO10163_SR_1	30
#define _ZUserInformationFieldISO10163_SR_1	13
#define _ZSR_APDUISO10163_SR_1	0
#define _ZRecordCompositionIdISO10163_SR_1	35
#define _ZAttributeListISO10163_SR_1	21
#define _ZRPNQueryISO10163_SR_1	17
#define _ZMaximumMessageSizeISO10163_SR_1	9
#define _ZOperatorISO10163_SR_1	23
#define _ZOptionsISO10163_SR_1	7
#define _ZDeleteResultSetResponseISO10163_SR_1	41
#define _ZNumberOfRecordsReturnedISO10163_SR_1	36
#define _ZTermISO10163_SR_1	22
#define _ZRecordCompositionISO10163_SR_1	34
#define _ZTHISEXTERNALISO10163_SR_1	3
#define _ZDiagRecISO10163_SR_1	33
#define _ZPresentRequestISO10163_SR_1	28
#define _ZObjectDescriptorISO10163_SR_1	2
#define _ZDatabaseNameISO10163_SR_1	45
#define _ZPreferredMessageSizeISO10163_SR_1	8
#define _ZNamePlusRecordISO10163_SR_1	31
#define _ZQueryISO10163_SR_1	16
#define _ZImplementationIdISO10163_SR_1	10
#define _ZDeleteSetStatusISO10163_SR_1	43
#define _ZNextResultSetPositionISO10163_SR_1	37

#ifndef	lint
#define encode_ISO10163__SR__1_SR__APDU(pe, top, len, buffer, parm) \
    enc_f(_ZSR_APDUISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_SR__APDU(pe, top, len, buffer, parm) \
    dec_f(_ZSR_APDUISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_SR__APDU(pe, top, len, buffer, parm) \
    prnt_f(_ZSR_APDUISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_SR__APDU_P    _ZSR_APDUISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_XSR__APDU(pe, top, len, buffer, parm) \
    enc_f(_ZXSR_APDUISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_XSR__APDU(pe, top, len, buffer, parm) \
    dec_f(_ZXSR_APDUISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_XSR__APDU(pe, top, len, buffer, parm) \
    prnt_f(_ZXSR_APDUISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_XSR__APDU_P    _ZXSR_APDUISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_ObjectDescriptor(pe, top, len, buffer, parm) \
    enc_f(_ZObjectDescriptorISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_ObjectDescriptor(pe, top, len, buffer, parm) \
    dec_f(_ZObjectDescriptorISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_ObjectDescriptor(pe, top, len, buffer, parm) \
    prnt_f(_ZObjectDescriptorISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_ObjectDescriptor_P    _ZObjectDescriptorISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_THISEXTERNAL(pe, top, len, buffer, parm) \
    enc_f(_ZTHISEXTERNALISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_THISEXTERNAL(pe, top, len, buffer, parm) \
    dec_f(_ZTHISEXTERNALISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_THISEXTERNAL(pe, top, len, buffer, parm) \
    prnt_f(_ZTHISEXTERNALISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_THISEXTERNAL_P    _ZTHISEXTERNALISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_InitialiseRequest(pe, top, len, buffer, parm) \
    enc_f(_ZInitialiseRequestISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_InitialiseRequest(pe, top, len, buffer, parm) \
    dec_f(_ZInitialiseRequestISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_InitialiseRequest(pe, top, len, buffer, parm) \
    prnt_f(_ZInitialiseRequestISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_InitialiseRequest_P    _ZInitialiseRequestISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_InitialiseResponse(pe, top, len, buffer, parm) \
    enc_f(_ZInitialiseResponseISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_InitialiseResponse(pe, top, len, buffer, parm) \
    dec_f(_ZInitialiseResponseISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_InitialiseResponse(pe, top, len, buffer, parm) \
    prnt_f(_ZInitialiseResponseISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_InitialiseResponse_P    _ZInitialiseResponseISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_ProtocolVersion(pe, top, len, buffer, parm) \
    enc_f(_ZProtocolVersionISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_ProtocolVersion(pe, top, len, buffer, parm) \
    dec_f(_ZProtocolVersionISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_ProtocolVersion(pe, top, len, buffer, parm) \
    prnt_f(_ZProtocolVersionISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_ProtocolVersion_P    _ZProtocolVersionISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_Options(pe, top, len, buffer, parm) \
    enc_f(_ZOptionsISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_Options(pe, top, len, buffer, parm) \
    dec_f(_ZOptionsISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_Options(pe, top, len, buffer, parm) \
    prnt_f(_ZOptionsISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_Options_P    _ZOptionsISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_PreferredMessageSize(pe, top, len, buffer, parm) \
    enc_f(_ZPreferredMessageSizeISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_PreferredMessageSize(pe, top, len, buffer, parm) \
    dec_f(_ZPreferredMessageSizeISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_PreferredMessageSize(pe, top, len, buffer, parm) \
    prnt_f(_ZPreferredMessageSizeISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_PreferredMessageSize_P    _ZPreferredMessageSizeISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_MaximumMessageSize(pe, top, len, buffer, parm) \
    enc_f(_ZMaximumMessageSizeISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_MaximumMessageSize(pe, top, len, buffer, parm) \
    dec_f(_ZMaximumMessageSizeISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_MaximumMessageSize(pe, top, len, buffer, parm) \
    prnt_f(_ZMaximumMessageSizeISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_MaximumMessageSize_P    _ZMaximumMessageSizeISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_ImplementationId(pe, top, len, buffer, parm) \
    enc_f(_ZImplementationIdISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_ImplementationId(pe, top, len, buffer, parm) \
    dec_f(_ZImplementationIdISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_ImplementationId(pe, top, len, buffer, parm) \
    prnt_f(_ZImplementationIdISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_ImplementationId_P    _ZImplementationIdISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_ImplementationName(pe, top, len, buffer, parm) \
    enc_f(_ZImplementationNameISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_ImplementationName(pe, top, len, buffer, parm) \
    dec_f(_ZImplementationNameISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_ImplementationName(pe, top, len, buffer, parm) \
    prnt_f(_ZImplementationNameISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_ImplementationName_P    _ZImplementationNameISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_ImplementationVersion(pe, top, len, buffer, parm) \
    enc_f(_ZImplementationVersionISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_ImplementationVersion(pe, top, len, buffer, parm) \
    dec_f(_ZImplementationVersionISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_ImplementationVersion(pe, top, len, buffer, parm) \
    prnt_f(_ZImplementationVersionISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_ImplementationVersion_P    _ZImplementationVersionISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_UserInformationField(pe, top, len, buffer, parm) \
    enc_f(_ZUserInformationFieldISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_UserInformationField(pe, top, len, buffer, parm) \
    dec_f(_ZUserInformationFieldISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_UserInformationField(pe, top, len, buffer, parm) \
    prnt_f(_ZUserInformationFieldISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_UserInformationField_P    _ZUserInformationFieldISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_Authentication(pe, top, len, buffer, parm) \
    enc_f(_ZAuthenticationISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_Authentication(pe, top, len, buffer, parm) \
    dec_f(_ZAuthenticationISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_Authentication(pe, top, len, buffer, parm) \
    prnt_f(_ZAuthenticationISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_Authentication_P    _ZAuthenticationISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_SearchRequest(pe, top, len, buffer, parm) \
    enc_f(_ZSearchRequestISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_SearchRequest(pe, top, len, buffer, parm) \
    dec_f(_ZSearchRequestISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_SearchRequest(pe, top, len, buffer, parm) \
    prnt_f(_ZSearchRequestISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_SearchRequest_P    _ZSearchRequestISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_Query(pe, top, len, buffer, parm) \
    enc_f(_ZQueryISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_Query(pe, top, len, buffer, parm) \
    dec_f(_ZQueryISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_Query(pe, top, len, buffer, parm) \
    prnt_f(_ZQueryISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_Query_P    _ZQueryISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_RPNQuery(pe, top, len, buffer, parm) \
    enc_f(_ZRPNQueryISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_RPNQuery(pe, top, len, buffer, parm) \
    dec_f(_ZRPNQueryISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_RPNQuery(pe, top, len, buffer, parm) \
    prnt_f(_ZRPNQueryISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_RPNQuery_P    _ZRPNQueryISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_RPNStructure(pe, top, len, buffer, parm) \
    enc_f(_ZRPNStructureISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_RPNStructure(pe, top, len, buffer, parm) \
    dec_f(_ZRPNStructureISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_RPNStructure(pe, top, len, buffer, parm) \
    prnt_f(_ZRPNStructureISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_RPNStructure_P    _ZRPNStructureISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_Operand(pe, top, len, buffer, parm) \
    enc_f(_ZOperandISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_Operand(pe, top, len, buffer, parm) \
    dec_f(_ZOperandISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_Operand(pe, top, len, buffer, parm) \
    prnt_f(_ZOperandISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_Operand_P    _ZOperandISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_AttributesPlusTerm(pe, top, len, buffer, parm) \
    enc_f(_ZAttributesPlusTermISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_AttributesPlusTerm(pe, top, len, buffer, parm) \
    dec_f(_ZAttributesPlusTermISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_AttributesPlusTerm(pe, top, len, buffer, parm) \
    prnt_f(_ZAttributesPlusTermISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_AttributesPlusTerm_P    _ZAttributesPlusTermISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_AttributeList(pe, top, len, buffer, parm) \
    enc_f(_ZAttributeListISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_AttributeList(pe, top, len, buffer, parm) \
    dec_f(_ZAttributeListISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_AttributeList(pe, top, len, buffer, parm) \
    prnt_f(_ZAttributeListISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_AttributeList_P    _ZAttributeListISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_Term(pe, top, len, buffer, parm) \
    enc_f(_ZTermISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_Term(pe, top, len, buffer, parm) \
    dec_f(_ZTermISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_Term(pe, top, len, buffer, parm) \
    prnt_f(_ZTermISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_Term_P    _ZTermISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_Operator(pe, top, len, buffer, parm) \
    enc_f(_ZOperatorISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_Operator(pe, top, len, buffer, parm) \
    dec_f(_ZOperatorISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_Operator(pe, top, len, buffer, parm) \
    prnt_f(_ZOperatorISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_Operator_P    _ZOperatorISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_AttributeElement(pe, top, len, buffer, parm) \
    enc_f(_ZAttributeElementISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_AttributeElement(pe, top, len, buffer, parm) \
    dec_f(_ZAttributeElementISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_AttributeElement(pe, top, len, buffer, parm) \
    prnt_f(_ZAttributeElementISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_AttributeElement_P    _ZAttributeElementISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_AttributeType(pe, top, len, buffer, parm) \
    enc_f(_ZAttributeTypeISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_AttributeType(pe, top, len, buffer, parm) \
    dec_f(_ZAttributeTypeISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_AttributeType(pe, top, len, buffer, parm) \
    prnt_f(_ZAttributeTypeISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_AttributeType_P    _ZAttributeTypeISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_AttributeValue(pe, top, len, buffer, parm) \
    enc_f(_ZAttributeValueISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_AttributeValue(pe, top, len, buffer, parm) \
    dec_f(_ZAttributeValueISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_AttributeValue(pe, top, len, buffer, parm) \
    prnt_f(_ZAttributeValueISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_AttributeValue_P    _ZAttributeValueISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_SearchResponse(pe, top, len, buffer, parm) \
    enc_f(_ZSearchResponseISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_SearchResponse(pe, top, len, buffer, parm) \
    dec_f(_ZSearchResponseISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_SearchResponse(pe, top, len, buffer, parm) \
    prnt_f(_ZSearchResponseISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_SearchResponse_P    _ZSearchResponseISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_PresentRequest(pe, top, len, buffer, parm) \
    enc_f(_ZPresentRequestISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_PresentRequest(pe, top, len, buffer, parm) \
    dec_f(_ZPresentRequestISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_PresentRequest(pe, top, len, buffer, parm) \
    prnt_f(_ZPresentRequestISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_PresentRequest_P    _ZPresentRequestISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_PresentResponse(pe, top, len, buffer, parm) \
    enc_f(_ZPresentResponseISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_PresentResponse(pe, top, len, buffer, parm) \
    dec_f(_ZPresentResponseISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_PresentResponse(pe, top, len, buffer, parm) \
    prnt_f(_ZPresentResponseISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_PresentResponse_P    _ZPresentResponseISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_Records(pe, top, len, buffer, parm) \
    enc_f(_ZRecordsISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_Records(pe, top, len, buffer, parm) \
    dec_f(_ZRecordsISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_Records(pe, top, len, buffer, parm) \
    prnt_f(_ZRecordsISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_Records_P    _ZRecordsISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_NamePlusRecord(pe, top, len, buffer, parm) \
    enc_f(_ZNamePlusRecordISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_NamePlusRecord(pe, top, len, buffer, parm) \
    dec_f(_ZNamePlusRecordISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_NamePlusRecord(pe, top, len, buffer, parm) \
    prnt_f(_ZNamePlusRecordISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_NamePlusRecord_P    _ZNamePlusRecordISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_DatabaseRecord(pe, top, len, buffer, parm) \
    enc_f(_ZDatabaseRecordISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_DatabaseRecord(pe, top, len, buffer, parm) \
    dec_f(_ZDatabaseRecordISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_DatabaseRecord(pe, top, len, buffer, parm) \
    prnt_f(_ZDatabaseRecordISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_DatabaseRecord_P    _ZDatabaseRecordISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_DiagRec(pe, top, len, buffer, parm) \
    enc_f(_ZDiagRecISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_DiagRec(pe, top, len, buffer, parm) \
    dec_f(_ZDiagRecISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_DiagRec(pe, top, len, buffer, parm) \
    prnt_f(_ZDiagRecISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_DiagRec_P    _ZDiagRecISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_RecordComposition(pe, top, len, buffer, parm) \
    enc_f(_ZRecordCompositionISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_RecordComposition(pe, top, len, buffer, parm) \
    dec_f(_ZRecordCompositionISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_RecordComposition(pe, top, len, buffer, parm) \
    prnt_f(_ZRecordCompositionISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_RecordComposition_P    _ZRecordCompositionISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_RecordCompositionId(pe, top, len, buffer, parm) \
    enc_f(_ZRecordCompositionIdISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_RecordCompositionId(pe, top, len, buffer, parm) \
    dec_f(_ZRecordCompositionIdISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_RecordCompositionId(pe, top, len, buffer, parm) \
    prnt_f(_ZRecordCompositionIdISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_RecordCompositionId_P    _ZRecordCompositionIdISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_NumberOfRecordsReturned(pe, top, len, buffer, parm) \
    enc_f(_ZNumberOfRecordsReturnedISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_NumberOfRecordsReturned(pe, top, len, buffer, parm) \
    dec_f(_ZNumberOfRecordsReturnedISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_NumberOfRecordsReturned(pe, top, len, buffer, parm) \
    prnt_f(_ZNumberOfRecordsReturnedISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_NumberOfRecordsReturned_P    _ZNumberOfRecordsReturnedISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_NextResultSetPosition(pe, top, len, buffer, parm) \
    enc_f(_ZNextResultSetPositionISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_NextResultSetPosition(pe, top, len, buffer, parm) \
    dec_f(_ZNextResultSetPositionISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_NextResultSetPosition(pe, top, len, buffer, parm) \
    prnt_f(_ZNextResultSetPositionISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_NextResultSetPosition_P    _ZNextResultSetPositionISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_PresentStatus(pe, top, len, buffer, parm) \
    enc_f(_ZPresentStatusISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_PresentStatus(pe, top, len, buffer, parm) \
    dec_f(_ZPresentStatusISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_PresentStatus(pe, top, len, buffer, parm) \
    prnt_f(_ZPresentStatusISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_PresentStatus_P    _ZPresentStatusISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_PreferredRecordSyntax(pe, top, len, buffer, parm) \
    enc_f(_ZPreferredRecordSyntaxISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_PreferredRecordSyntax(pe, top, len, buffer, parm) \
    dec_f(_ZPreferredRecordSyntaxISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_PreferredRecordSyntax(pe, top, len, buffer, parm) \
    prnt_f(_ZPreferredRecordSyntaxISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_PreferredRecordSyntax_P    _ZPreferredRecordSyntaxISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_DeleteResultSetRequest(pe, top, len, buffer, parm) \
    enc_f(_ZDeleteResultSetRequestISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_DeleteResultSetRequest(pe, top, len, buffer, parm) \
    dec_f(_ZDeleteResultSetRequestISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_DeleteResultSetRequest(pe, top, len, buffer, parm) \
    prnt_f(_ZDeleteResultSetRequestISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_DeleteResultSetRequest_P    _ZDeleteResultSetRequestISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_DeleteResultSetResponse(pe, top, len, buffer, parm) \
    enc_f(_ZDeleteResultSetResponseISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_DeleteResultSetResponse(pe, top, len, buffer, parm) \
    dec_f(_ZDeleteResultSetResponseISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_DeleteResultSetResponse(pe, top, len, buffer, parm) \
    prnt_f(_ZDeleteResultSetResponseISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_DeleteResultSetResponse_P    _ZDeleteResultSetResponseISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_ListStatuses(pe, top, len, buffer, parm) \
    enc_f(_ZListStatusesISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_ListStatuses(pe, top, len, buffer, parm) \
    dec_f(_ZListStatusesISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_ListStatuses(pe, top, len, buffer, parm) \
    prnt_f(_ZListStatusesISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_ListStatuses_P    _ZListStatusesISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_DeleteSetStatus(pe, top, len, buffer, parm) \
    enc_f(_ZDeleteSetStatusISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_DeleteSetStatus(pe, top, len, buffer, parm) \
    dec_f(_ZDeleteSetStatusISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_DeleteSetStatus(pe, top, len, buffer, parm) \
    prnt_f(_ZDeleteSetStatusISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_DeleteSetStatus_P    _ZDeleteSetStatusISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_ReferenceId(pe, top, len, buffer, parm) \
    enc_f(_ZReferenceIdISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_ReferenceId(pe, top, len, buffer, parm) \
    dec_f(_ZReferenceIdISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_ReferenceId(pe, top, len, buffer, parm) \
    prnt_f(_ZReferenceIdISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_ReferenceId_P    _ZReferenceIdISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_DatabaseName(pe, top, len, buffer, parm) \
    enc_f(_ZDatabaseNameISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_DatabaseName(pe, top, len, buffer, parm) \
    dec_f(_ZDatabaseNameISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_DatabaseName(pe, top, len, buffer, parm) \
    prnt_f(_ZDatabaseNameISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_DatabaseName_P    _ZDatabaseNameISO10163_SR_1, &_ZISO10163_SR_1_mod

#define encode_ISO10163__SR__1_ResultSetId(pe, top, len, buffer, parm) \
    enc_f(_ZResultSetIdISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char *) parm)

#define decode_ISO10163__SR__1_ResultSetId(pe, top, len, buffer, parm) \
    dec_f(_ZResultSetIdISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer, (char **) parm)

#define print_ISO10163__SR__1_ResultSetId(pe, top, len, buffer, parm) \
    prnt_f(_ZResultSetIdISO10163_SR_1, &_ZISO10163_SR_1_mod, pe, top, len, buffer)
#define print_ISO10163__SR__1_ResultSetId_P    _ZResultSetIdISO10163_SR_1, &_ZISO10163_SR_1_mod


#endif   /* lint */

                /* OCTET STRING */
#define	type_ISO10163__SR__1_ObjectDescriptor	qbuf
#define	free_ISO10163__SR__1_ObjectDescriptor	qb_free

                /* BIT STRING */
#define	type_ISO10163__SR__1_ProtocolVersion	PElement
#define	free_ISO10163__SR__1_ProtocolVersion	pe_free

                /* BIT STRING */
#define	type_ISO10163__SR__1_Options	PElement
#define	bits_ISO10163__SR__1_Options	"\020\01search\02present\03delSet"
#define	bit_ISO10163__SR__1_Options_search	0
#define	bit_ISO10163__SR__1_Options_present	1
#define	bit_ISO10163__SR__1_Options_delSet	2
#define	free_ISO10163__SR__1_Options	pe_free

                /* VisibleString */
#define	type_ISO10163__SR__1_ImplementationId	type_UNIV_VisibleString
#define	free_ISO10163__SR__1_ImplementationId	free_UNIV_VisibleString

                /* VisibleString */
#define	type_ISO10163__SR__1_ImplementationName	type_UNIV_VisibleString
#define	free_ISO10163__SR__1_ImplementationName	free_UNIV_VisibleString

                /* VisibleString */
#define	type_ISO10163__SR__1_ImplementationVersion	type_UNIV_VisibleString
#define	free_ISO10163__SR__1_ImplementationVersion	free_UNIV_VisibleString

                /* THISEXTERNAL */
#define	type_ISO10163__SR__1_UserInformationField	type_ISO10163__SR__1_THISEXTERNAL
#define	free_ISO10163__SR__1_UserInformationField	free_ISO10163__SR__1_THISEXTERNAL

                /* OCTET STRING */
#define	type_ISO10163__SR__1_Term	qbuf
#define	free_ISO10163__SR__1_Term	qb_free

                /* THISEXTERNAL */
#define	type_ISO10163__SR__1_DatabaseRecord	type_ISO10163__SR__1_THISEXTERNAL
#define	free_ISO10163__SR__1_DatabaseRecord	free_ISO10163__SR__1_THISEXTERNAL

                /* VisibleString */
#define	type_ISO10163__SR__1_RecordCompositionId	type_UNIV_VisibleString
#define	free_ISO10163__SR__1_RecordCompositionId	free_UNIV_VisibleString

                /* OBJECT IDENTIFIER */
#define	type_ISO10163__SR__1_PreferredRecordSyntax	OIDentifier
#define	free_ISO10163__SR__1_PreferredRecordSyntax	oid_free

                /* OCTET STRING */
#define	type_ISO10163__SR__1_ReferenceId	qbuf
#define	free_ISO10163__SR__1_ReferenceId	qb_free

                /* VisibleString */
#define	type_ISO10163__SR__1_DatabaseName	type_UNIV_VisibleString
#define	free_ISO10163__SR__1_DatabaseName	free_UNIV_VisibleString

                /* VisibleString */
#define	type_ISO10163__SR__1_ResultSetId	type_UNIV_VisibleString
#define	free_ISO10163__SR__1_ResultSetId	free_UNIV_VisibleString

                /* CHOICE */
struct type_ISO10163__SR__1_SR__APDU {
    int         offset;
#define	type_ISO10163__SR__1_SR__APDU_initialiseRequest	1
#define	type_ISO10163__SR__1_SR__APDU_initialiseResponse	2
#define	type_ISO10163__SR__1_SR__APDU_searchRequest	3
#define	type_ISO10163__SR__1_SR__APDU_searchResponse	4
#define	type_ISO10163__SR__1_SR__APDU_presentRequest	5
#define	type_ISO10163__SR__1_SR__APDU_presentResponse	6
#define	type_ISO10163__SR__1_SR__APDU_deleteResultSetRequest	7
#define	type_ISO10163__SR__1_SR__APDU_deleteResultSetResponse	8

    union {
                        /* ANY */
        PE      initialiseRequest;

                        /* ANY */
        PE      initialiseResponse;

                        /* ANY */
        PE      searchRequest;

                        /* ANY */
        PE      searchResponse;

                        /* ANY */
        PE      presentRequest;

                        /* ANY */
        PE      presentResponse;

                        /* ANY */
        PE      deleteResultSetRequest;

                        /* ANY */
        PE      deleteResultSetResponse;
    }       un;
};
#define	free_ISO10163__SR__1_SR__APDU(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZSR_APDUISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* CHOICE */
struct type_ISO10163__SR__1_XSR__APDU {
    int         offset;
#define	type_ISO10163__SR__1_XSR__APDU_initialiseRequest	1
#define	type_ISO10163__SR__1_XSR__APDU_initialiseResponse	2
#define	type_ISO10163__SR__1_XSR__APDU_searchRequest	3
#define	type_ISO10163__SR__1_XSR__APDU_searchResponse	4
#define	type_ISO10163__SR__1_XSR__APDU_presentRequest	5
#define	type_ISO10163__SR__1_XSR__APDU_presentResponse	6
#define	type_ISO10163__SR__1_XSR__APDU_deleteResultSetRequest	7
#define	type_ISO10163__SR__1_XSR__APDU_deleteResultSetResponse	8

    union {
                        /* InitialiseRequest */
        struct type_ISO10163__SR__1_InitialiseRequest *initialiseRequest;

                        /* InitialiseResponse */
        struct type_ISO10163__SR__1_InitialiseResponse *initialiseResponse;

                        /* SearchRequest */
        struct type_ISO10163__SR__1_SearchRequest *searchRequest;

                        /* SearchResponse */
        struct type_ISO10163__SR__1_SearchResponse *searchResponse;

                        /* PresentRequest */
        struct type_ISO10163__SR__1_PresentRequest *presentRequest;

                        /* PresentResponse */
        struct type_ISO10163__SR__1_PresentResponse *presentResponse;

                        /* DeleteResultSetRequest */
        struct type_ISO10163__SR__1_DeleteResultSetRequest *deleteResultSetRequest;

                        /* DeleteResultSetResponse */
        struct type_ISO10163__SR__1_DeleteResultSetResponse *deleteResultSetResponse;
    }       un;
};
#define	free_ISO10163__SR__1_XSR__APDU(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZXSR_APDUISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* SEQUENCE */
struct type_ISO10163__SR__1_THISEXTERNAL {
    integer     optionals;
#define	opt_ISO10163__SR__1_THISEXTERNAL_indirect__reference (000000001)

                    /* OBJECT IDENTIFIER */
    OID     direct__reference;

                    /* INTEGER */
    integer     indirect__reference;

                    /* ObjectDescriptor */
    struct type_UNIV_ObjectDescriptor *data__value__descriptor;

                    /* CHOICE */
    struct choice_ISO10163__SR__1_0 {
        int         offset;
#define	choice_ISO10163__SR__1_0_single__ASN1__type	1
#define	choice_ISO10163__SR__1_0_octet__aligned	2
#define	choice_ISO10163__SR__1_0_arbitrary	3

        union {
                            /* ANY */
            PE      single__ASN1__type;

                            /* OCTET STRING */
            struct qbuf *octet__aligned;

                            /* BIT STRING */
            PE      arbitrary;
        }       un;
    } *element_ISO10163__SR__1_0;
};
#define	free_ISO10163__SR__1_THISEXTERNAL(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZTHISEXTERNALISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* SEQUENCE */
struct type_ISO10163__SR__1_InitialiseRequest {
                    /* ReferenceId */
    struct type_ISO10163__SR__1_ReferenceId *referenceId;

                    /* ProtocolVersion */
    struct type_ISO10163__SR__1_ProtocolVersion *protocolVersion;

                    /* Options */
    struct type_ISO10163__SR__1_Options *options;

                    /* PreferredMessageSize */
    struct type_ISO10163__SR__1_PreferredMessageSize *preferredMessageSize;

                    /* MaximumMessageSize */
    struct type_ISO10163__SR__1_MaximumMessageSize *maximumMessageSize;

                    /* ANY */
    PE      authentication;

                    /* ImplementationId */
    struct type_ISO10163__SR__1_ImplementationId *implementationId;

                    /* ImplementationName */
    struct type_ISO10163__SR__1_ImplementationName *implementationName;

                    /* ImplementationVersion */
    struct type_ISO10163__SR__1_ImplementationVersion *implementationVersion;

                    /* UserInformationField */
    struct type_ISO10163__SR__1_UserInformationField *userInformationField;
};
#define	free_ISO10163__SR__1_InitialiseRequest(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZInitialiseRequestISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* SEQUENCE */
struct type_ISO10163__SR__1_InitialiseResponse {
                    /* ReferenceId */
    struct type_ISO10163__SR__1_ReferenceId *referenceId;

                    /* ProtocolVersion */
    struct type_ISO10163__SR__1_ProtocolVersion *protocolVersion;

                    /* Options */
    struct type_ISO10163__SR__1_Options *options;

                    /* PreferredMessageSize */
    struct type_ISO10163__SR__1_PreferredMessageSize *preferredMessageSize;

                    /* MaximumMessageSize */
    struct type_ISO10163__SR__1_MaximumMessageSize *maximumMessageSize;

                    /* BOOLEAN */
    char    initializationStatus;

                    /* ImplementationId */
    struct type_ISO10163__SR__1_ImplementationId *implementationId;

                    /* ImplementationName */
    struct type_ISO10163__SR__1_ImplementationName *implementationName;

                    /* ImplementationVersion */
    struct type_ISO10163__SR__1_ImplementationVersion *implementationVersion;

                    /* UserInformationField */
    struct type_ISO10163__SR__1_UserInformationField *userInformationField;
};
#define	free_ISO10163__SR__1_InitialiseResponse(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZInitialiseResponseISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* INTEGER */
struct type_ISO10163__SR__1_PreferredMessageSize {
                    /* INTEGER */
    integer     parm;
};
#define	free_ISO10163__SR__1_PreferredMessageSize(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZPreferredMessageSizeISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* INTEGER */
struct type_ISO10163__SR__1_MaximumMessageSize {
                    /* INTEGER */
    integer     parm;
};
#define	free_ISO10163__SR__1_MaximumMessageSize(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZMaximumMessageSizeISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* SEQUENCE */
struct type_ISO10163__SR__1_Authentication {
                    /* VisibleString */
    struct	qbuf	*user;

                    /* VisibleString */
    struct	qbuf	*password;

                    /* VisibleString */
    struct	qbuf	*account;
};
#define	free_ISO10163__SR__1_Authentication(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZAuthenticationISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* SEQUENCE */
struct type_ISO10163__SR__1_SearchRequest {
                    /* ReferenceId */
    struct type_ISO10163__SR__1_ReferenceId *referenceId;

                    /* INTEGER */
    integer     smallSetUpperBound;

                    /* INTEGER */
    integer     largeSetLowerBound;

                    /* INTEGER */
    integer     mediumSetPresentNumber;

                    /* BOOLEAN */
    char    replaceIndicator;

                    /* VisibleString */
    struct	qbuf	*proposedResultSetId;

                    /* SEQUENCE OF */
    struct element_ISO10163__SR__1_1 {
                        /* DatabaseName */
        struct type_ISO10163__SR__1_DatabaseName *DatabaseName;

        struct element_ISO10163__SR__1_1 *next;
    } *databaseId;

                    /* RecordComposition */
    struct type_ISO10163__SR__1_RecordComposition *smallSetRecordComposition;

                    /* RecordComposition */
    struct type_ISO10163__SR__1_RecordComposition *mediumSetRecordComposition;

                    /* PreferredRecordSyntax */
    struct type_ISO10163__SR__1_PreferredRecordSyntax *preferredRecordSyntax;

                    /* Query */
    struct type_ISO10163__SR__1_Query *query;
};
#define	free_ISO10163__SR__1_SearchRequest(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZSearchRequestISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* CHOICE */
struct type_ISO10163__SR__1_Query {
    int         offset;
#define	type_ISO10163__SR__1_Query_privateQuery	1
#define	type_ISO10163__SR__1_Query_rpnQuery	2
#define	type_ISO10163__SR__1_Query_iso8777query	3

    union {
                        /* ANY */
        PE      privateQuery;

                        /* RPNQuery */
        struct type_ISO10163__SR__1_RPNQuery *rpnQuery;

                        /* OCTET STRING */
        struct qbuf *iso8777query;
    }       un;
};
#define	free_ISO10163__SR__1_Query(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZQueryISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* SEQUENCE */
struct type_ISO10163__SR__1_RPNQuery {
                    /* OBJECT IDENTIFIER */
    OID     attributeSetId;

                    /* RPNStructure */
    struct type_ISO10163__SR__1_RPNStructure *element_ISO10163__SR__1_2;
};
#define	free_ISO10163__SR__1_RPNQuery(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZRPNQueryISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* CHOICE */
struct type_ISO10163__SR__1_RPNStructure {
    int         offset;
#define	type_ISO10163__SR__1_RPNStructure_1	1
#define	type_ISO10163__SR__1_RPNStructure_2	2

    union {
                        /* Operand */
        struct type_ISO10163__SR__1_Operand *choice_ISO10163__SR__1_1;

                        /* SEQUENCE */
        struct element_ISO10163__SR__1_3 {
                            /* RPNStructure */
            struct type_ISO10163__SR__1_RPNStructure *element_ISO10163__SR__1_4;

                            /* RPNStructure */
            struct type_ISO10163__SR__1_RPNStructure *element_ISO10163__SR__1_5;

                            /* Operator */
            struct type_ISO10163__SR__1_Operator *element_ISO10163__SR__1_6;
        } *choice_ISO10163__SR__1_2;
    }       un;
};
#define	free_ISO10163__SR__1_RPNStructure(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZRPNStructureISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* CHOICE */
struct type_ISO10163__SR__1_Operand {
    int         offset;
#define	type_ISO10163__SR__1_Operand_1	1
#define	type_ISO10163__SR__1_Operand_2	2

    union {
                        /* AttributesPlusTerm */
        struct type_ISO10163__SR__1_AttributesPlusTerm *choice_ISO10163__SR__1_3;

                        /* ResultSetId */
        struct type_ISO10163__SR__1_ResultSetId *choice_ISO10163__SR__1_4;
    }       un;
};
#define	free_ISO10163__SR__1_Operand(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZOperandISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* SEQUENCE */
struct type_ISO10163__SR__1_AttributesPlusTerm {
                    /* AttributeList */
    struct type_ISO10163__SR__1_AttributeList *element_ISO10163__SR__1_7;

                    /* Term */
    struct type_ISO10163__SR__1_Term *element_ISO10163__SR__1_8;
};
#define	free_ISO10163__SR__1_AttributesPlusTerm(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZAttributesPlusTermISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* SEQUENCE OF */
struct type_ISO10163__SR__1_AttributeList {
                        /* AttributeElement */
        struct type_ISO10163__SR__1_AttributeElement *AttributeElement;

        struct type_ISO10163__SR__1_AttributeList *next;
};
#define	free_ISO10163__SR__1_AttributeList(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZAttributeListISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* CHOICE */
struct type_ISO10163__SR__1_Operator {
    int         offset;
#define	type_ISO10163__SR__1_Operator_and	1
#define	type_ISO10163__SR__1_Operator_or	2
#define	type_ISO10163__SR__1_Operator_and__not	3

    union {
                        /* NULL */
        char    and;

                        /* NULL */
        char    or;

                        /* NULL */
        char    and__not;
    }       un;
};
#define	free_ISO10163__SR__1_Operator(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZOperatorISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* SEQUENCE */
struct type_ISO10163__SR__1_AttributeElement {
                    /* AttributeType */
    struct type_ISO10163__SR__1_AttributeType *element_ISO10163__SR__1_9;

                    /* AttributeValue */
    struct type_ISO10163__SR__1_AttributeValue *element_ISO10163__SR__1_10;
};
#define	free_ISO10163__SR__1_AttributeElement(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZAttributeElementISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* INTEGER */
struct type_ISO10163__SR__1_AttributeType {
                    /* INTEGER */
    integer     parm;
};
#define	free_ISO10163__SR__1_AttributeType(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZAttributeTypeISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* INTEGER */
struct type_ISO10163__SR__1_AttributeValue {
                    /* INTEGER */
    integer     parm;
};
#define	free_ISO10163__SR__1_AttributeValue(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZAttributeValueISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* SEQUENCE */
struct type_ISO10163__SR__1_SearchResponse {
    integer     optionals;
#define	opt_ISO10163__SR__1_SearchResponse_resultSetStatus (000000001)

                    /* ReferenceId */
    struct type_ISO10163__SR__1_ReferenceId *referenceId;

                    /* INTEGER */
    integer     numberOfRecordsFound;

                    /* NumberOfRecordsReturned */
    struct type_ISO10163__SR__1_NumberOfRecordsReturned *numberOfRecordsReturned;

                    /* NextResultSetPosition */
    struct type_ISO10163__SR__1_NextResultSetPosition *nextResultSetPosition;

                    /* BOOLEAN */
    char    searchStatus;

                    /* INTEGER */
    integer     resultSetStatus;
#define	int_ISO10163__SR__1_resultSetStatus_subset	1
#define	int_ISO10163__SR__1_resultSetStatus_interim	2
#define	int_ISO10163__SR__1_resultSetStatus_none	3

                    /* PresentStatus */
    struct type_ISO10163__SR__1_PresentStatus *presentStatus;

                    /* Records */
    struct type_ISO10163__SR__1_Records *records;
};
#define	free_ISO10163__SR__1_SearchResponse(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZSearchResponseISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* SEQUENCE */
struct type_ISO10163__SR__1_PresentRequest {
                    /* ReferenceId */
    struct type_ISO10163__SR__1_ReferenceId *referenceId;

                    /* ResultSetId */
    struct type_ISO10163__SR__1_ResultSetId *resultSetId;

                    /* INTEGER */
    integer     resultSetStartPoint;

                    /* INTEGER */
    integer     numberOfRecordsRequested;

                    /* RecordComposition */
    struct type_ISO10163__SR__1_RecordComposition *element_ISO10163__SR__1_11;

                    /* PreferredRecordSyntax */
    struct type_ISO10163__SR__1_PreferredRecordSyntax *preferredRecordSyntax;
};
#define	free_ISO10163__SR__1_PresentRequest(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZPresentRequestISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* SEQUENCE */
struct type_ISO10163__SR__1_PresentResponse {
                    /* ReferenceId */
    struct type_ISO10163__SR__1_ReferenceId *referenceId;

                    /* NumberOfRecordsReturned */
    struct type_ISO10163__SR__1_NumberOfRecordsReturned *numberOfRecordsReturned;

                    /* NextResultSetPosition */
    struct type_ISO10163__SR__1_NextResultSetPosition *nextResultSetPosition;

                    /* PresentStatus */
    struct type_ISO10163__SR__1_PresentStatus *presentStatus;

                    /* Records */
    struct type_ISO10163__SR__1_Records *records;
};
#define	free_ISO10163__SR__1_PresentResponse(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZPresentResponseISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* CHOICE */
struct type_ISO10163__SR__1_Records {
    int         offset;
#define	type_ISO10163__SR__1_Records_dataBaseOrSurDiagnostics	1
#define	type_ISO10163__SR__1_Records_nonSurrogateDiagnostic	2

    union {
                        /* SEQUENCE OF */
        struct element_ISO10163__SR__1_12 {
                            /* NamePlusRecord */
            struct type_ISO10163__SR__1_NamePlusRecord *NamePlusRecord;

            struct element_ISO10163__SR__1_12 *next;
        } *dataBaseOrSurDiagnostics;

                        /* DiagRec */
        struct type_ISO10163__SR__1_DiagRec *nonSurrogateDiagnostic;
    }       un;
};
#define	free_ISO10163__SR__1_Records(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZRecordsISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* SEQUENCE */
struct type_ISO10163__SR__1_NamePlusRecord {
                    /* DatabaseName */
    struct type_ISO10163__SR__1_DatabaseName *element_ISO10163__SR__1_13;

                    /* CHOICE */
    struct choice_ISO10163__SR__1_5 {
        int         offset;
#define	choice_ISO10163__SR__1_5_databaseRecord	1
#define	choice_ISO10163__SR__1_5_surrogateDiagnostic	2

        union {
                            /* DatabaseRecord */
            struct type_ISO10163__SR__1_DatabaseRecord *databaseRecord;

                            /* DiagRec */
            struct type_ISO10163__SR__1_DiagRec *surrogateDiagnostic;
        }       un;
    } *element_ISO10163__SR__1_14;
};
#define	free_ISO10163__SR__1_NamePlusRecord(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZNamePlusRecordISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* SEQUENCE */
struct type_ISO10163__SR__1_DiagRec {
                    /* OBJECT IDENTIFIER */
    OID     diagnosticSetId;

                    /* INTEGER */
    integer     condition;

                    /* VisibleString */
    struct	qbuf	*addinfo;
};
#define	free_ISO10163__SR__1_DiagRec(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZDiagRecISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* CHOICE */
struct type_ISO10163__SR__1_RecordComposition {
    int         offset;
#define	type_ISO10163__SR__1_RecordComposition_generic	1
#define	type_ISO10163__SR__1_RecordComposition_databaseSpecific	2

    union {
                        /* RecordCompositionId */
        struct type_ISO10163__SR__1_RecordCompositionId *generic;

                        /* SEQUENCE OF */
        struct element_ISO10163__SR__1_15 {
                            /* SEQUENCE */
            struct element_ISO10163__SR__1_17 {
                                /* DatabaseName */
                struct type_ISO10163__SR__1_DatabaseName *element_ISO10163__SR__1_18;

                                /* RecordCompositionId */
                struct type_ISO10163__SR__1_RecordCompositionId *element_ISO10163__SR__1_19;
            } *element_ISO10163__SR__1_16;

            struct element_ISO10163__SR__1_15 *next;
        } *databaseSpecific;
    }       un;
};
#define	free_ISO10163__SR__1_RecordComposition(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZRecordCompositionISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* INTEGER */
struct type_ISO10163__SR__1_NumberOfRecordsReturned {
                    /* INTEGER */
    integer     parm;
};
#define	free_ISO10163__SR__1_NumberOfRecordsReturned(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZNumberOfRecordsReturnedISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* INTEGER */
struct type_ISO10163__SR__1_NextResultSetPosition {
                    /* INTEGER */
    integer     parm;
};
#define	free_ISO10163__SR__1_NextResultSetPosition(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZNextResultSetPositionISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* INTEGER */
struct type_ISO10163__SR__1_PresentStatus {
                    /* INTEGER */
    integer     parm;
#define	int_ISO10163__SR__1_PresentStatus_success	0
#define	int_ISO10163__SR__1_PresentStatus_partial__1	1
#define	int_ISO10163__SR__1_PresentStatus_partial__2	2
#define	int_ISO10163__SR__1_PresentStatus_partial__4	4
#define	int_ISO10163__SR__1_PresentStatus_failure	5
};
#define	free_ISO10163__SR__1_PresentStatus(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZPresentStatusISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* SEQUENCE */
struct type_ISO10163__SR__1_DeleteResultSetRequest {
                    /* ReferenceId */
    struct type_ISO10163__SR__1_ReferenceId *referenceId;

                    /* INTEGER */
    integer     deleteSetFunction;
#define	int_ISO10163__SR__1_deleteSetFunction_list	0
#define	int_ISO10163__SR__1_deleteSetFunction_all	1

                    /* SEQUENCE OF */
    struct element_ISO10163__SR__1_20 {
                        /* ResultSetId */
        struct type_ISO10163__SR__1_ResultSetId *ResultSetId;

        struct element_ISO10163__SR__1_20 *next;
    } *resultSetList;
};
#define	free_ISO10163__SR__1_DeleteResultSetRequest(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZDeleteResultSetRequestISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* SEQUENCE */
struct type_ISO10163__SR__1_DeleteResultSetResponse {
    integer     optionals;
#define	opt_ISO10163__SR__1_DeleteResultSetResponse_numberNotDeleted (000000001)

                    /* ReferenceId */
    struct type_ISO10163__SR__1_ReferenceId *referenceId;

                    /* DeleteSetStatus */
    struct type_ISO10163__SR__1_DeleteSetStatus *deleteOperationStatus;

                    /* ListStatuses */
    struct type_ISO10163__SR__1_ListStatuses *deleteListStatuses;

                    /* INTEGER */
    integer     numberNotDeleted;

                    /* ListStatuses */
    struct type_ISO10163__SR__1_ListStatuses *bulkStatuses;

                    /* VisibleString */
    struct	qbuf	*deleteMessage;
};
#define	free_ISO10163__SR__1_DeleteResultSetResponse(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZDeleteResultSetResponseISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* SEQUENCE OF */
struct type_ISO10163__SR__1_ListStatuses {
                        /* SEQUENCE */
        struct element_ISO10163__SR__1_22 {
                            /* ResultSetId */
            struct type_ISO10163__SR__1_ResultSetId *element_ISO10163__SR__1_23;

                            /* DeleteSetStatus */
            struct type_ISO10163__SR__1_DeleteSetStatus *element_ISO10163__SR__1_24;
        } *element_ISO10163__SR__1_21;

        struct type_ISO10163__SR__1_ListStatuses *next;
};
#define	free_ISO10163__SR__1_ListStatuses(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZListStatusesISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)

                /* INTEGER */
struct type_ISO10163__SR__1_DeleteSetStatus {
                    /* INTEGER */
    integer     parm;
#define	int_ISO10163__SR__1_DeleteSetStatus_success	0
#define	int_ISO10163__SR__1_DeleteSetStatus_resultSetDidNotExist	1
#define	int_ISO10163__SR__1_DeleteSetStatus_previouslyDeletedByTarget	2
#define	int_ISO10163__SR__1_DeleteSetStatus_systemProblemAtTarget	3
#define	int_ISO10163__SR__1_DeleteSetStatus_accessNotAllowed	4
#define	int_ISO10163__SR__1_DeleteSetStatus_resourceControl	6
#define	int_ISO10163__SR__1_DeleteSetStatus_bulkDeleteNotSupported	7
#define	int_ISO10163__SR__1_DeleteSetStatus_notAllResultSetsDeletedOnBulkDelete	8
#define	int_ISO10163__SR__1_DeleteSetStatus_notAllRequestedResultSetsDeleted	9
};
#define	free_ISO10163__SR__1_DeleteSetStatus(parm)\
	(void) fre_obj((char *) parm, _ZISO10163_SR_1_mod.md_dtab[_ZDeleteSetStatusISO10163_SR_1], &_ZISO10163_SR_1_mod, 1)
#endif
