/*
 *   This file was automatically generated by version 1.7 of cextract.
 *   Manual editing not recommended.
 *
 *   Created: Fri Jan 28 14:33:32 1994
 */

extern srPktStatus closeConnection ( int fd );
extern srPktStatus ConnectToDatabaseServer ( connectionParameters *coninfo, int *Fd, elemDesc *struct_top );
extern srPktStatus CreateListener ( connectionParameters *coninfo, int *Fd, elemDesc *struct_top );
extern Boolean CreateListener2 ( int fd, int *Fd, elemDesc *struct_top );
extern void set_no_fork ( int i );
extern void wait_for_children ( void );

