/*
 *   This file was automatically generated by version 1.7 of cextract.
 *   Manual editing not recommended.
 *
 *   Created: Tue Jan 25 16:21:07 1994
 */
#ifndef __CEXTRACT__
#if __STDC__

extern Boolean ConvertMarcTextEntryToLabel ( Marcrecord *rec );
extern void dump_record ( Marcrecord *rec );
extern void Init_Marclabel ( Record_label *label );
extern void log_start ( char *cp, int max );
extern Marcrecord * ReadMarcText ( octetString *os );
extern Boolean ReadMarcTextEntry ( Marcrecord *rec, octetString *os, unsigned long *pos );
extern void UpdateMarcLabelDirectoryFormat ( Marcrecord *rec );
extern octetString * WriteMarcText ( Marcrecord *rec );
extern void WriteMarcTextEntry ( Marcdata *entry, char *data, unsigned long *length );

#else /* __STDC__ */

extern Boolean ConvertMarcTextEntryToLabel (/* Marcrecord *rec */);
extern void dump_record (/* Marcrecord *rec */);
extern void Init_Marclabel (/* Record_label *label */);
extern void log_start (/* char *cp, int max */);
extern Marcrecord * ReadMarcText (/* octetString *os */);
extern Boolean ReadMarcTextEntry (/* Marcrecord *rec, octetString *os, unsigned long *pos */);
extern void UpdateMarcLabelDirectoryFormat (/* Marcrecord *rec */);
extern octetString * WriteMarcText (/* Marcrecord *rec */);
extern void WriteMarcTextEntry (/* Marcdata *entry, char *data, unsigned long *length */);

#endif /* __STDC__ */
#endif /* __CEXTRACT__ */
