
/*
 * sr_structcodec.h,v 1.1 1994/01/28 17:06:04 franktor Exp
 *
 * sr_structcodec.h
 *
*/

#ifndef SR_STRUCTCODEC_H
#define SR_STRUCTCODEC_H

#include <sr-general.h>
#include <sr-api.h>
#ifndef VMS
/* #include <high/eapi.h> */
#include <high/structcodec.h>
#else
/* #include <eapi.h> */
#include <structcodec.h>
#endif

typedef enum srPktStatus { srPktStat_ok, srPktStat_failed, srPktStat_packetReady,
			   srPktStat_connectionBroken, srPktStat_connectionClosed } srPktStatus; 

typedef enum EapiConnectTypes {
   connect_tcpip = 0, connect_pipe = 1, connect_none = 2,
#ifndef VMS
   connect_tcpip_multi = 3
#else
   connect_tcpip_monolith = 3
#endif
} EapiConnectTypes;

typedef struct connectionParameters { 
   EapiConnectTypes    	method;
   union
   {
      struct 
      {
	 char 		*address;
	 unsigned int 	port;
#ifdef VMS
	 int		*socket;
#endif
      } ip;
      char 		*program;
      int 		dummy;
   } u;
} connectionParameters;


#endif /* SR_STRUCTCODEC_H */
