/*
 *   This file was automatically generated by version 1.7 of cextract.
 *   Manual editing not recommended.
 *
 *   Created: Fri Jan 28 14:33:26 1994
 */
#ifndef __CEXTRACT__
#if __STDC__

extern int pe2str ( PE pe, char **buf, int *len );
extern void TCPIP_SetError ( char *err );
extern void _TCPIP_AbortAssociation ( int ref );
extern Boolean _TCPIP_CheckIncommingQ ( int *ref, qElem **qelem );
extern void _TCPIP_ClearAssociation ( int ref );
extern void _TCPIP_Close ( void );
extern Boolean _TCPIP_CreateListener ( SR_Address *where, Boolean (*accept)( int, SR_protocolVersion, LowerLayerKind, presContext *pci ) );
extern void _TCPIP_GetSelectFD_SET ( fd_set **rs, fd_set **ws );
extern void _TCPIP_HandleFileDescriptors ( fd_set *rfds, fd_set *wfds, fd_set *efds );
extern Boolean _TCPIP_Initialise ( LogLevel logLevel, struct _iobuf *log, void (*Abort)(int) );
extern char *_TCPIP_LastError ( void );
extern Boolean _TCPIP_OpenAssociation ( SR_Address *to, presContext **pci, int *remoteRef );
extern SROperation _TCPIP_PollNetwork ( struct timeval *timeout, int *ref );
extern SRCloseRequest *_TCPIP_ReadCloseRequest ( int ref );
extern SRCloseResponse *_TCPIP_ReadCloseResponse ( int ref );
extern SRDeleteResultSetRequest *_TCPIP_ReadDeleteResultSetRequest ( int ref );
extern SRDeleteResultSetResponse *_TCPIP_ReadDeleteResultSetResponse ( int ref );
extern SRInitialiseRequest *_TCPIP_ReadInitialiseRequest ( int ref );
extern SRInitialiseResponse *_TCPIP_ReadInitialiseResponse ( int ref );
extern SRPresentRequest *_TCPIP_ReadPresentRequest ( int ref );
extern SRPresentResponse *_TCPIP_ReadPresentResponse ( int ref );
extern SRSearchRequest *_TCPIP_ReadSearchRequest ( int ref );
extern SRSearchResponse *_TCPIP_ReadSearchResponse ( int ref );
extern int _TCPIP_Select ( struct timeval *timeout, int nfds, fd_set *rfds, fd_set *wfds, fd_set *efds );
extern Boolean _TCPIP_SendCloseRequest ( int ref, SRCloseRequest *crq );
extern Boolean _TCPIP_SendCloseResponse ( int ref, SRCloseResponse *crp );
extern Boolean _TCPIP_SendDeleteResultSetRequest ( int ref, SRDeleteResultSetRequest * ir );
extern Boolean _TCPIP_SendDeleteResultSetResponse ( int ref, SRDeleteResultSetResponse * ir );
extern Boolean _TCPIP_SendInitialiseRequest ( int ref, SRInitialiseRequest * ir );
extern Boolean _TCPIP_SendInitialiseResponse ( int ref, SRInitialiseResponse * ir );
extern Boolean _TCPIP_SendPresentRequest ( int ref, SRPresentRequest * ir );
extern Boolean _TCPIP_SendPresentResponse ( int ref, SRPresentResponse * ir );
extern Boolean _TCPIP_SendSearchRequest ( int ref, SRSearchRequest * ir );
extern Boolean _TCPIP_SendSearchResponse ( int ref, SRSearchResponse * ir );
extern Boolean _TCPIP_Verify ( void );
extern Boolean _TCPIP_VerifyAssociation ( int ref );

#else /* __STDC__ */

extern int pe2str (/* PE pe, char **buf, int *len */);
extern void TCPIP_SetError (/* char *err */);
extern void _TCPIP_AbortAssociation (/* int ref */);
extern Boolean _TCPIP_CheckIncommingQ (/* int *ref, qElem **qelem */);
extern void _TCPIP_ClearAssociation (/* int ref */);
extern void _TCPIP_Close (/* void */);
extern Boolean _TCPIP_CreateListener (/* SR_Address *where, Boolean (*accept)( int, SR_protocolVersion, LowerLayerKind, presContext *pci ) */);
extern void _TCPIP_GetSelectFD_SET (/* fd_set **rs, fd_set **ws */);
extern void _TCPIP_HandleFileDescriptors (/* fd_set *rfds, fd_set *wfds, fd_set *efds */);
extern Boolean _TCPIP_Initialise (/* LogLevel logLevel, struct _iobuf *log, void (*Abort)(int) */);
extern char *_TCPIP_LastError (/* void */);
extern Boolean _TCPIP_OpenAssociation (/* SR_Address *to, presContext **pci, int *remoteRef */);
extern SROperation _TCPIP_PollNetwork (/* struct timeval *timeout, int *ref */);
extern SRCloseRequest *_TCPIP_ReadCloseRequest (/* int ref */);
extern SRCloseResponse *_TCPIP_ReadCloseResponse (/* int ref */);
extern SRDeleteResultSetRequest *_TCPIP_ReadDeleteResultSetRequest (/* int ref */);
extern SRDeleteResultSetResponse *_TCPIP_ReadDeleteResultSetResponse (/* int ref */);
extern SRInitialiseRequest *_TCPIP_ReadInitialiseRequest (/* int ref */);
extern SRInitialiseResponse *_TCPIP_ReadInitialiseResponse (/* int ref */);
extern SRPresentRequest *_TCPIP_ReadPresentRequest (/* int ref */);
extern SRPresentResponse *_TCPIP_ReadPresentResponse (/* int ref */);
extern SRSearchRequest *_TCPIP_ReadSearchRequest (/* int ref */);
extern SRSearchResponse *_TCPIP_ReadSearchResponse (/* int ref */);
extern int _TCPIP_Select (/* struct timeval *timeout, int nfds, fd_set *rfds, fd_set *wfds, fd_set *efds */);
extern Boolean _TCPIP_SendCloseRequest (/* int ref, SRCloseRequest *crq */);
extern Boolean _TCPIP_SendCloseResponse (/* int ref, SRCloseResponse *crp */);
extern Boolean _TCPIP_SendDeleteResultSetRequest (/* int ref, SRDeleteResultSetRequest * ir */);
extern Boolean _TCPIP_SendDeleteResultSetResponse (/* int ref, SRDeleteResultSetResponse * ir */);
extern Boolean _TCPIP_SendInitialiseRequest (/* int ref, SRInitialiseRequest * ir */);
extern Boolean _TCPIP_SendInitialiseResponse (/* int ref, SRInitialiseResponse * ir */);
extern Boolean _TCPIP_SendPresentRequest (/* int ref, SRPresentRequest * ir */);
extern Boolean _TCPIP_SendPresentResponse (/* int ref, SRPresentResponse * ir */);
extern Boolean _TCPIP_SendSearchRequest (/* int ref, SRSearchRequest * ir */);
extern Boolean _TCPIP_SendSearchResponse (/* int ref, SRSearchResponse * ir */);
extern Boolean _TCPIP_Verify (/* void */);
extern Boolean _TCPIP_VerifyAssociation (/* int ref */);

#endif /* __STDC__ */
#endif /* __CEXTRACT__ */
