/*
 *   This file was automatically generated by version 1.7 of cextract.
 *   Manual editing not recommended.
 *
 *   Created: Fri Jan 28 14:33:28 1994
 */
#ifndef __CEXTRACT__
#if __STDC__

extern Boolean FD_ISZERO ( fd_set *x );
extern Boolean SendPE ( int ref, PE pe, char *operation, char *kind );
extern void _ISODE_AbortAssociation ( int ref );
extern Boolean _ISODE_CheckIncommingQ ( int *ref, qElem **qelem );
extern void _ISODE_ClearAssociation ( int ref );
extern Boolean _ISODE_CreateListener ( SR_Address *where, Boolean (*accept)( int, SR_protocolVersion, LowerLayerKind, presContext *pci ) );
extern fd_set *_ISODE_GetSelectFD_SET ( void );
extern char *_ISODE_LastError ( void );
extern Boolean _ISODE_OpenAssociation ( SR_Address *to, presContext **pci, int *remoteRef );
extern SROperation _ISODE_PollNetwork ( struct timeval *timeout, int nfds, fd_set *rfds, fd_set *wfds, fd_set *efds, int *ref );
extern SRCloseRequest *_ISODE_ReadCloseRequest ( int ref );
extern SRCloseResponse *_ISODE_ReadCloseResponse ( int ref );
extern SRDeleteResultSetRequest *_ISODE_ReadDeleteResultSetRequest ( int ref );
extern SRDeleteResultSetResponse *_ISODE_ReadDeleteResultSetResponse ( int ref );
extern SRInitialiseRequest *_ISODE_ReadInitialiseRequest ( int ref );
extern SRInitialiseResponse *_ISODE_ReadInitialiseResponse ( int ref );
extern SRPresentRequest *_ISODE_ReadPresentRequest ( int ref );
extern SRPresentResponse *_ISODE_ReadPresentResponse ( int ref );
extern SRSearchRequest *_ISODE_ReadSearchRequest ( int ref );
extern SRSearchResponse *_ISODE_ReadSearchResponse ( int ref );
extern Boolean _ISODE_SendCloseRequest ( int ref, SRCloseRequest *crq );
extern Boolean _ISODE_SendCloseResponse ( int ref, SRCloseResponse *crp );
extern Boolean _ISODE_SendDeleteResultSetRequest ( int ref, SRDeleteResultSetRequest * ir );
extern Boolean _ISODE_SendDeleteResultSetResponse ( int ref, SRDeleteResultSetResponse * ir );
extern Boolean _ISODE_SendInitialiseRequest ( int ref, SRInitialiseRequest * ir );
extern Boolean _ISODE_SendInitialiseResponse ( int ref, SRInitialiseResponse * ir );
extern Boolean _ISODE_SendPresentRequest ( int ref, SRPresentRequest * ir );
extern Boolean _ISODE_SendPresentResponse ( int ref, SRPresentResponse * ir );
extern Boolean _ISODE_SendSearchRequest ( int ref, SRSearchRequest * ir );
extern Boolean _ISODE_SendSearchResponse ( int ref, SRSearchResponse * ir );
extern Boolean _ISODE_VerifyAssociation ( int ref );
extern void _OSI_Close ( void );
extern Boolean _OSI_Initialise ( LogLevel logLevel, struct _iobuf *log, void (*Abort)(int) );
extern Boolean _OSI_Verify ( void );

#else /* __STDC__ */

extern Boolean FD_ISZERO (/* fd_set *x */);
extern Boolean SendPE (/* int ref, PE pe, char *operation, char *kind */);
extern void _ISODE_AbortAssociation (/* int ref */);
extern Boolean _ISODE_CheckIncommingQ (/* int *ref, qElem **qelem */);
extern void _ISODE_ClearAssociation (/* int ref */);
extern Boolean _ISODE_CreateListener (/* SR_Address *where, Boolean (*accept)( int, SR_protocolVersion, LowerLayerKind, presContext *pci ) */);
extern fd_set *_ISODE_GetSelectFD_SET (/* void */);
extern char *_ISODE_LastError (/* void */);
extern Boolean _ISODE_OpenAssociation (/* SR_Address *to, presContext **pci, int *remoteRef */);
extern SROperation _ISODE_PollNetwork (/* struct timeval *timeout, int nfds, fd_set *rfds, fd_set *wfds, fd_set *efds, int *ref */);
extern SRCloseRequest *_ISODE_ReadCloseRequest (/* int ref */);
extern SRCloseResponse *_ISODE_ReadCloseResponse (/* int ref */);
extern SRDeleteResultSetRequest *_ISODE_ReadDeleteResultSetRequest (/* int ref */);
extern SRDeleteResultSetResponse *_ISODE_ReadDeleteResultSetResponse (/* int ref */);
extern SRInitialiseRequest *_ISODE_ReadInitialiseRequest (/* int ref */);
extern SRInitialiseResponse *_ISODE_ReadInitialiseResponse (/* int ref */);
extern SRPresentRequest *_ISODE_ReadPresentRequest (/* int ref */);
extern SRPresentResponse *_ISODE_ReadPresentResponse (/* int ref */);
extern SRSearchRequest *_ISODE_ReadSearchRequest (/* int ref */);
extern SRSearchResponse *_ISODE_ReadSearchResponse (/* int ref */);
extern Boolean _ISODE_SendCloseRequest (/* int ref, SRCloseRequest *crq */);
extern Boolean _ISODE_SendCloseResponse (/* int ref, SRCloseResponse *crp */);
extern Boolean _ISODE_SendDeleteResultSetRequest (/* int ref, SRDeleteResultSetRequest * ir */);
extern Boolean _ISODE_SendDeleteResultSetResponse (/* int ref, SRDeleteResultSetResponse * ir */);
extern Boolean _ISODE_SendInitialiseRequest (/* int ref, SRInitialiseRequest * ir */);
extern Boolean _ISODE_SendInitialiseResponse (/* int ref, SRInitialiseResponse * ir */);
extern Boolean _ISODE_SendPresentRequest (/* int ref, SRPresentRequest * ir */);
extern Boolean _ISODE_SendPresentResponse (/* int ref, SRPresentResponse * ir */);
extern Boolean _ISODE_SendSearchRequest (/* int ref, SRSearchRequest * ir */);
extern Boolean _ISODE_SendSearchResponse (/* int ref, SRSearchResponse * ir */);
extern Boolean _ISODE_VerifyAssociation (/* int ref */);
extern void _OSI_Close (/* void */);
extern Boolean _OSI_Initialise (/* LogLevel logLevel, struct _iobuf *log, void (*Abort)(int) */);
extern Boolean _OSI_Verify (/* void */);

#endif /* __STDC__ */
#endif /* __CEXTRACT__ */
