/* prototypes for some ISODE functions */

#ifndef	isode_proto_H_
#define isode_proto_H_

#include <isode/psap.h>
#include <ansidef.h>

OID	ode2oid		ARGDCL((const char *));
int	oid_cmp		ARGDCL((OID, OID)), elem_cmp ();
OID	oid_cpy		ARGDCL((OID));
int	oid_free	ARGDCL((OID));
char   *oid2ode_aux	ARGDCL((OID, int));
char   *sprintoid	ARGDCL((OID));
OID	str2oid		ARGDCL((const char *));

PE	pe_alloc	ARGDCL((PElementClass, PElementForm, PElementID));
int	pe_free		ARGDCL((PE));
int	pe_cmp		ARGDCL((PE, PE));
PE	pe_cpy		ARGDCL((PE));
int	pe_pullup ();
PE	pe_expunge	ARGDCL((PE, PE));
int	pe_extract ();

PE	str2pe		ARGDCL((char *, int, int *, int *));
PE	qb2pe		ARGDCL((struct qbuf *, int, int, int*));

#endif /* isode_proto_H_ */
