/*
 * marctext.h,v 1.1 1994/01/28 17:06:46 franktor Exp
 */

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <memory.h>
#include <marc.h>
#include <sr-general.h>
#ifndef __CEXTRACT__
#include <marcproto.h>
#include <utilproto.h>
#endif
#ifdef __sun__
#include <sunos.h>
#endif

#define MARC_TEXT_LINE_START '*'  /* If this should be portable, numbers */
#define MARC_TEXT_LINE_END   '^'  /* should probably be used instead.    */
