/*
 *   This file was automatically generated by version 1.7 of cextract.
 *   Manual editing not recommended.
 *
 *   Created: Fri Jan 28 14:33:24 1994
 */
#ifndef __CEXTRACT__
#if __STDC__

extern AttributeElement *decode_AttributeElement ( struct _AttributeElement *req );
extern AttributeList *decode_AttributeList ( struct _AttributeList *req );
extern AttributesPlusTerm *decode_AttributesPlusTerm ( struct _AttributesPlusTerm *req );
extern struct Authentication *decode_Authentication ( PE pe );
extern SRDeleteResultSetRequest *decode_DeleteResultSetRequest ( struct _DeleteResultSetRequest *req );
extern SRDeleteResultSetResponse *decode_DeleteResultSetResponse ( struct _DeleteResultSetResponse *req );
extern DiagRec *decode_DiagRec ( struct _DiagRec *req );
extern EXTERN *decode_EXTERN ( struct Extern *e );
extern SRInitialiseRequest *decode_InitialiseRequest ( struct _InitialiseRequest *req );
extern SRInitialiseResponse *decode_InitialiseResponse ( struct _InitialiseResponse *req );
extern ListStatuses *decode_ListStatuses ( struct _ListStatuses *req );
extern NamePlusRecord *decode_NamePlusRecord ( struct _NamePlusRecord *req );
extern Operand *decode_Operand ( struct _Operand *req );
extern Operator decode_Operator ( struct _Operator *req );
extern SRPresentRequest *decode_PresentRequest ( struct _PresentRequest *req );
extern SRPresentResponse *decode_PresentResponse ( struct _PresentResponse *req );
extern struct Query *decode_Query ( struct _Query *req );
extern RCNameAndId *decode_RCDatabaseSpecific ( struct _RCDatabaseSpecific *req );
extern RecordComposition *decode_RecordComposition ( struct _RecordComposition *req );
extern Records *decode_Records ( struct _Records *req );
extern ResultSetList *decode_ResultSetList ( struct _ResultSetList *req );
extern RPNQuery *decode_RPNQuery ( struct _RPNQuery *req );
extern RPNStructure *decode_RPNStructure ( struct _RPNStructure *req );
extern SRSearchRequest *decode_SearchRequest ( struct _SearchRequest *req );
extern SRSearchResponse *decode_SearchResponse ( struct _SearchResponse *req );
extern char **decode_SeqOfDatabaseName ( struct _SeqOf_DatabaseName *req, int *size );
extern struct _AttributeElement *encode_AttributeElement ( AttributeElement *req );
extern struct _AttributeList *encode_AttributeList ( AttributeList *req );
extern struct _AttributesPlusTerm *encode_AttributesPlusTerm ( AttributesPlusTerm *req );
extern PE encode_Authentication ( Authentication *req );
extern struct _DeleteResultSetRequest *encode_DeleteResultSetRequest ( SRDeleteResultSetRequest *req );
extern struct _DeleteResultSetResponse *encode_DeleteResultSetResponse ( SRDeleteResultSetResponse *req );
extern struct _DiagRec *encode_DiagRec ( DiagRec *req );
extern struct Extern *encode_EXTERN ( EXTERN *e );
extern struct _InitialiseRequest *encode_InitialiseRequest ( SRInitialiseRequest *req );
extern struct _InitialiseResponse *encode_InitialiseResponse ( SRInitialiseResponse *req );
extern struct _ListStatuses *encode_ListStatuses ( ListStatuses *req );
extern struct _NamePlusRecord *encode_NamePlusRecord ( NamePlusRecord *req );
extern struct _Operand *encode_Operand ( Operand *req );
extern struct _Operator *encode_Operator ( Operator req );
extern struct _PresentRequest *encode_PresentRequest ( SRPresentRequest *req );
extern struct _PresentResponse *encode_PresentResponse ( SRPresentResponse *req );
extern struct _Query *encode_Query ( Query *req );
extern struct _RCDatabaseSpecific *encode_RCDatabaseSpecific ( RCNameAndId *sr_rc );
extern struct _RecordComposition *encode_RecordComposition ( RecordComposition *req );
extern struct _Records *encode_Records ( Records *req );
extern struct _ResultSetList *encode_ResultSetList ( ResultSetList *req );
extern struct _RPNQuery *encode_RPNQuery ( RPNQuery *req );
extern struct _RPNStructure *encode_RPNStructure ( RPNStructure *req );
extern struct _SearchRequest *encode_SearchRequest ( SRSearchRequest *req );
extern struct _SearchResponse *encode_SearchResponse ( SRSearchResponse *req );
extern struct _SeqOf_DatabaseName *encode_SeqOfDatabaseName ( char **req, int size );

#else /* __STDC__ */

extern AttributeElement *decode_AttributeElement (/* struct _AttributeElement *req */);
extern AttributeList *decode_AttributeList (/* struct _AttributeList *req */);
extern AttributesPlusTerm *decode_AttributesPlusTerm (/* struct _AttributesPlusTerm *req */);
extern struct Authentication *decode_Authentication (/* PE pe */);
extern SRDeleteResultSetRequest *decode_DeleteResultSetRequest (/* struct _DeleteResultSetRequest *req */);
extern SRDeleteResultSetResponse *decode_DeleteResultSetResponse (/* struct _DeleteResultSetResponse *req */);
extern DiagRec *decode_DiagRec (/* struct _DiagRec *req */);
extern EXTERN *decode_EXTERN (/* struct Extern *e */);
extern SRInitialiseRequest *decode_InitialiseRequest (/* struct _InitialiseRequest *req */);
extern SRInitialiseResponse *decode_InitialiseResponse (/* struct _InitialiseResponse *req */);
extern ListStatuses *decode_ListStatuses (/* struct _ListStatuses *req */);
extern NamePlusRecord *decode_NamePlusRecord (/* struct _NamePlusRecord *req */);
extern Operand *decode_Operand (/* struct _Operand *req */);
extern Operator decode_Operator (/* struct _Operator *req */);
extern SRPresentRequest *decode_PresentRequest (/* struct _PresentRequest *req */);
extern SRPresentResponse *decode_PresentResponse (/* struct _PresentResponse *req */);
extern struct Query *decode_Query (/* struct _Query *req */);
extern RCNameAndId *decode_RCDatabaseSpecific (/* struct _RCDatabaseSpecific *req */);
extern RecordComposition *decode_RecordComposition (/* struct _RecordComposition *req */);
extern Records *decode_Records (/* struct _Records *req */);
extern ResultSetList *decode_ResultSetList (/* struct _ResultSetList *req */);
extern RPNQuery *decode_RPNQuery (/* struct _RPNQuery *req */);
extern RPNStructure *decode_RPNStructure (/* struct _RPNStructure *req */);
extern SRSearchRequest *decode_SearchRequest (/* struct _SearchRequest *req */);
extern SRSearchResponse *decode_SearchResponse (/* struct _SearchResponse *req */);
extern char **decode_SeqOfDatabaseName (/* struct _SeqOf_DatabaseName *req, int *size */);
extern struct _AttributeElement *encode_AttributeElement (/* AttributeElement *req */);
extern struct _AttributeList *encode_AttributeList (/* AttributeList *req */);
extern struct _AttributesPlusTerm *encode_AttributesPlusTerm (/* AttributesPlusTerm *req */);
extern PE encode_Authentication (/* Authentication *req */);
extern struct _DeleteResultSetRequest *encode_DeleteResultSetRequest (/* SRDeleteResultSetRequest *req */);
extern struct _DeleteResultSetResponse *encode_DeleteResultSetResponse (/* SRDeleteResultSetResponse *req */);
extern struct _DiagRec *encode_DiagRec (/* DiagRec *req */);
extern struct Extern *encode_EXTERN (/* EXTERN *e */);
extern struct _InitialiseRequest *encode_InitialiseRequest (/* SRInitialiseRequest *req */);
extern struct _InitialiseResponse *encode_InitialiseResponse (/* SRInitialiseResponse *req */);
extern struct _ListStatuses *encode_ListStatuses (/* ListStatuses *req */);
extern struct _NamePlusRecord *encode_NamePlusRecord (/* NamePlusRecord *req */);
extern struct _Operand *encode_Operand (/* Operand *req */);
extern struct _Operator *encode_Operator (/* Operator req */);
extern struct _PresentRequest *encode_PresentRequest (/* SRPresentRequest *req */);
extern struct _PresentResponse *encode_PresentResponse (/* SRPresentResponse *req */);
extern struct _Query *encode_Query (/* Query *req */);
extern struct _RCDatabaseSpecific *encode_RCDatabaseSpecific (/* RCNameAndId *sr_rc */);
extern struct _RecordComposition *encode_RecordComposition (/* RecordComposition *req */);
extern struct _Records *encode_Records (/* Records *req */);
extern struct _ResultSetList *encode_ResultSetList (/* ResultSetList *req */);
extern struct _RPNQuery *encode_RPNQuery (/* RPNQuery *req */);
extern struct _RPNStructure *encode_RPNStructure (/* RPNStructure *req */);
extern struct _SearchRequest *encode_SearchRequest (/* SRSearchRequest *req */);
extern struct _SearchResponse *encode_SearchResponse (/* SRSearchResponse *req */);
extern struct _SeqOf_DatabaseName *encode_SeqOfDatabaseName (/* char **req, int size */);

#endif /* __STDC__ */
#endif /* __CEXTRACT__ */
