#include <sr-util.h>

RecordComposition *new_RecordComposition ( Boolean isGeneric, char *genericComposition )
{
   RecordComposition *rc;

   if ( (rc = (RecordComposition *) smalloc ( sizeof (RecordComposition) )) )
      if ( (rc->genericRecordComposition = isGeneric) )
	 rc->u.generic = safe_strdup ( genericComposition );
   return rc;
}
