/*
 *   This file was automatically generated by version 1.7 of cextract.
 *   Manual editing not recommended.
 *
 *   Created: Fri Jan 28 14:33:34 1994
 */
#ifndef __CEXTRACT__
#if __STDC__

extern long strntol ( char *str, char **ptr, int base, int length );

#else /* __STDC__ */

extern long strntol (/* char *str, char **ptr, int base, int length */);

#endif /* __STDC__ */
#endif /* __CEXTRACT__ */
