/*
 *   This file was automatically generated by version 1.7 of cextract.
 *   Manual editing not recommended.
 *
 *   Created: Sun Jan 30 16:07:58 1994
 */

extern void abortClient ( Client *client );
extern void abortServerConnection ( Client *client, ServerConnection *conn );
extern void abort_connection ( Client *client, ServerConnection *conn );
extern void add_client ( int fd );
extern DiagRec * ConvertFromHlvlDiagRec ( HlvlDiagRec *e_diag );
extern NamePlusRecord * ConvertFromHlvlNamePlusRecord ( HlvlNamePlusRecord *e_rec );
extern Records * ConvertFromHlvlRecords ( HlvlRecords * e_rec );
extern HlvlDiagRec * ConvertToHlvlDiagRec ( DiagRec *diag );
extern HlvlNamePlusRecord * ConvertToHlvlNamePlusRecord ( NamePlusRecord *rec );
extern Query * ConvertToHlvlQuery ( Query *q );
extern HlvlRecords * ConvertToHlvlRecords ( Records *rec );
extern Query * copyQuery ( Query *q );
extern RCNameAndId * copyRCNameAndId ( RCNameAndId *in );
extern RecordComposition * copyRecordComposition ( RecordComposition *comp );
extern Client * fd2client ( int fd );
extern Client * find_client ( int ref );
extern Client * find_client_from_SR_ref ( int ref );
extern ServerConnection * find_connection ( Client *client, int ref );
extern ServerConnection * find_connection_from_SR_ref ( Client *client, int ref );
extern void free_packet ( HC_Packet *pk );
extern void handleAbortResponse ( int ref );
extern void handleCloseRequest ( int fd, int ref, HC_CloseRequest *req );
extern void handleCloseResponse ( int ref );
extern void handleDeleteResultSetRequest ( int fd, int ref, HC_DeleteResultSetRequest *req );
extern void handleDeleteResultSetResponse ( int ref );
extern void handleInitialiseResponse ( int ref );
extern void handleOpenRequest ( int fd, int ref, HC_OpenRequest *req );
extern void handlePresentRequest ( int fd, int ref, HC_PresentRequest *req );
extern void handlePresentResponse ( int ref );
extern void handleSearchRequest ( int fd, int ref, HC_SearchRequest *req );
extern void handleSearchResponse ( int ref );
extern int main ( int argc, char **argv );
extern void remove_client ( Client *client );
extern void remove_server_connection ( Client *client, ServerConnection *conn );
extern void sendErrorResponse ( Client *client, ServerConnection *conn, const char *message, Boolean abort );
extern Boolean setupResponseHandler ( int ref, Client **client, ServerConnection **conn, const char *response_name );

