/*
 * asn1.c,v 1.1 1994/01/28 17:07:01 franktor Exp
 *
 * asn1.c
 *
 * Tester for SR ASN.1
 *
*/

#include <stdio.h>
#include <isode/acsap.h>

#include "ISO10163-SR-1-types.h"
#include "SR.h"

typedef enum Boolean {True=1, False=0} Boolean; 

struct Query	tst;


main ()
{
	PE		pe;
	PS		ps;

	ps = ps_alloc ( std_open );
	std_setup ( ps, stderr );

/* */

	tst.offset = 2;
	tst.rpnQuery = malloc ( sizeof ( struct RPNQuery ) );
	tst.rpnQuery->attributeSetId = str2oid ( "0.1.1" );
	tst.rpnQuery->rpnStructure.offset = 1;
	tst.rpnQuery->rpnStructure.operand = malloc ( sizeof ( struct Operand ) );
	tst.rpnQuery->rpnStructure.operand->offset = 1;
	tst.rpnQuery->rpnStructure.operand->attributesPlusTerm = 0;

/* malloc ( sizeof ( struct AttributesPlusTerm ) );
	tst.rpnQuery->rpnStructure.operand.attributesPlusTerm->attributeList = malloc ( sizeof ( struct AttributeList ) );
	tst.rpnQuery->rpnStructure.operand.attributesPlusTerm->attributeList->attributeElement = 
*/	
	
	
	

	printf ( "result of encode=%d\n", encode_ISO10163__SR__1_Query ( &pe, 1, 0, NULLCP, &tst ) );

/* */

	pe2pl ( ps, pe );
	ps_free ( ps );
}
