/*
 *   This file was automatically generated by version 1.7 of cextract.
 *   Manual editing not recommended.
 *
 *   Created: Tue Jan 25 16:21:05 1994
 */
#ifndef __CEXTRACT__
#if __STDC__

extern void FreeMarc ( Marcrecord *rec );
extern void FreeMarcEntries ( Marcdata *entry );

#else /* __STDC__ */

extern void FreeMarc (/* Marcrecord *rec */);
extern void FreeMarcEntries (/* Marcdata *entry */);

#endif /* __STDC__ */
#endif /* __CEXTRACT__ */
