/*
 *   This file was automatically generated by version 1.7 of cextract.
 *   Manual editing not recommended.
 *
 *   Created: Fri Jan 28 14:33:41 1994
 */

extern Boolean AcceptAssoc ( int ref, presContext *pc );
extern Client * add_client ( int ref );
extern ClientResultSetList * add_ClientResultSet ( Client *client, ClientDatabaseServer *server, char *name );
extern void add_converter ( const char *func, const char *abstract, const char *transfer );
extern void add_databases ( DatabaseServer *dbs, char *name );
extern DatabaseServerConnection * add_DatabaseServerConnection ( DatabaseServer *dbs );
extern ClientDatabaseServer * add_DatabaseServerConnectionToClient ( DatabaseServerConnection *connection, Client *client );
extern void add_database_server ( DatabaseServer *dbs );
extern void add_supported_syntaxes ( DatabaseServer *dbs, char *name );
extern void CheckConversion ( Client *client );
extern void close_servers ( Client *client );
extern ClientDatabaseServer * connectClientToDatabaseServer ( Client *client, DatabaseServer *server );
extern DiagRec * ConvertFromEapiDiagRec ( EapiDiagRec *e_diag );
extern NamePlusRecord * ConvertFromEapiNamePlusRecord ( Client *client, EapiNamePlusRecord *e_rec );
extern Records * ConvertFromEapiRecords ( Client *client, EapiRecords * e_rec );
extern Query * ConvertToEapiQuery ( Query *q );
extern octetString * CopyOctStr ( octetString *os );
extern ClientDatabaseServer * Database2ClientDatabaseServer ( Client *client, DatabaseName *database );
extern Client *fd2client ( int fd );
extern ClientDatabaseServer * fd2ClientDatabaseServer ( Client *client, int fd );
extern DatabaseServerConnection * fd2DatabaseServerConnection ( int fd );
extern Client *find_client ( int ref );
extern ClientResultSetList * find_ClientResultSet ( Client *client, char *name );
extern DatabaseServerConnection * find_DatabaseServerConnection ( DatabaseServer *dbs );
extern ClientDatabaseServer * FirstClientDatabaseServer ( Client *client, ClientServerConnectionStatuses state );
extern ClientDatabaseServer * FirstNonsingleClientDatabaseServer ( Client *client, ClientServerConnectionStatuses state );
extern ClientDatabaseServer * FirstSingleClientDatabaseServer ( Client *client, ClientServerConnectionStatuses state );
extern void free_EapiPacket ( EapiPacket *ep );
extern void free_resultsets ( ClientResultSetList *resultset );
extern config_vars get_config_var ( struct _iobuf *fp );
extern char *get_word ( struct _iobuf *fp );
extern void handleAbort ( int ref );
extern void handleCloseRequest ( int ref );
extern void handleCloseResponse ( Client *client, EapiPacket *ep );
extern void handleDeleteResultSetRequest ( int ref );
extern void handleDeleteResultSetResponse ( Client *client, EapiPacket *ep );
extern void handleInitialiseRequest ( int ref );
extern void handleInterruptResponse ( Client *client, EapiPacket *packet );
extern void handleOpenResponse ( Client *client, EapiPacket *ep );
extern void handlePresentRequest ( int ref );
extern void handlePresentResponse ( Client *client, EapiPacket *ep );
extern void handleSearchRequest ( int ref );
extern void handleSearchResponse ( Client *client, EapiPacket *ep );
extern void initialise ( void );
extern void main ( int argc, char **argv );
extern void parse_args ( int argc, char **argv );
extern void read_localname ( struct _iobuf *fp );
extern void read_server ( struct _iobuf *fp );
extern Boolean ReceiveDBPacket ( EapiPacket *packet );
extern octetString * Record_CopyOctStr ( Client *client, octetString *os );
extern Client *ref2client ( int ref );
extern void remove_client ( Client *client );
extern void remove_ClientResultSet ( Client *client, ClientResultSetList *resultset );
extern void remove_DatabaseServerConnection ( DatabaseServerConnection *dbsc, Boolean Close );
extern void remove_DatabaseServerConnectionToClient ( ClientDatabaseServer *cdbs, Client *client, Boolean Close );
extern Boolean SendDBPacket ( EapiPacket *packet, Client *client, DatabaseServerConnection *connection );
extern void sendPresentResponseDiagnostic ( Client *client, int condition, char *addinfo );
extern void sendSearchResponseDiagnostic ( Client *client, int condition, char *addinfo );
extern void send_close_response ( Client *client );
extern void send_delete_request_failure ( Client *client, DeleteSetStatus fault, char *msg );
extern void send_one_delete_request ( Client *client );
extern void set_relevant_bits ( void );

