/*
 * init.c,v 1.1 1994/01/28 17:06:25 franktor Exp
 *
 * This is just an experimental main() to test out the functions.
 */

#include <sr-general.h>
#include <sr-converter.h>
#include "marc/marc.h"
#include "config.h"
#ifndef __CEXTRACT__
#include <marcproto.h>
#include "proto.h"
#endif

void init_converter() {
  RegisterConverter("MARC", convert_in, "NORMARC-data", (void *) ReadMarcData);
  RegisterConverter("MARC", convert_out, "NORMARC-data", (void *) WriteMarcData);
  RegisterConverter("MARC", convert_in, "NORMARC-text", (void *) ReadMarcText);
  RegisterConverter("MARC", convert_out, "NORMARC-text", (void *) WriteMarcText);
  RegisterClassFree("MARC", (void *) FreeMarc);
}
