/*
 *   This file was automatically generated by version 1.7 of cextract.
 *   Manual editing not recommended.
 *
 *   Created: Wed Jul  7 11:57:23 1993
 */

extern Boolean a2b ( char *ch );
extern Boolean AcceptAssociation ( int refnr, presContext * pc );
extern char * b2str ( Boolean tf );
extern void changeLogfile ( void );
extern void CompareRecordCompositions ( int refnr, RecordComposition *rec1, RecordComposition *rec2, char *what );
extern void CompRPN ( int refnr, RPNStructure * rpn1, RPNStructure * rpn2 );
extern SRCloseResponse * GetSRCloseRespA ( char *fnm );
extern SRCloseResponse * GetSRCloseRespM ( void );
extern SRDeleteResultSetRequest * GetSRDelRSReqValuesA ( char *fnm );
extern SRDeleteResultSetRequest * GetSRDelRSReqValuesM ( void );
extern SRDeleteResultSetResponse * GetSRDelRSRspValuesA ( char *fnm );
extern SRDeleteResultSetResponse * GetSRDelRSRspValuesM ( void );
extern SRInitialiseRequest * GetSRInitReqValuesA ( char *fnm );
extern SRInitialiseRequest * GetSRInitReqValuesM ( void );
extern SRInitialiseResponse * GetSRInitRespValuesA ( char *fnm );
extern SRInitialiseResponse * GetSRInitRespValuesM ( void );
extern SRPresentRequest * GetSRPresReqValuesA ( char *fnm );
extern SRPresentRequest * GetSRPresReqValuesM ( void );
extern SRPresentResponse * GetSRPresRspValuesA ( char *fnm );
extern SRPresentResponse * GetSRPresRspValuesM ( void );
extern SRSearchRequest * GetSRSrchReqValuesA ( char *fnm );
extern SRSearchRequest * GetSRSrchReqValuesM ( void );
extern SRSearchResponse * GetSRSrchRespValuesA ( char *fnm );
extern SRSearchResponse * GetSRSrchRespValuesM ( void );
extern char gnu_getchar ( char *prompt );
extern char *gnu_gets ( char *prompt, struct _iobuf *from, char *buf, int len );
extern void gnu_gets_setup ( void );
extern int main ( void );
extern RCNameAndId * ReadDatabaseSpecificRecordCompositions ( char *name );
extern Boolean RecASP_O ( int refnr );
extern Boolean RecASP_T ( int refnr );
extern Boolean SendASP_O ( int refnr );
extern Boolean SendASP_T ( int refnr );
extern void ShowRecordComposition ( int refnr, RecordComposition *rec );
extern void ShowRPN ( RPNStructure * rpnS );
extern void ShowSRDelResSetReqV ( int refnr, SRDeleteResultSetRequest * req );
extern void ShowSRDelResSetRespV ( int refnr, SRDeleteResultSetResponse * rsp );
extern void ShowSRInitReqV ( int refnr, SRInitialiseRequest * req );
extern void ShowSRInitRespV ( int refnr, SRInitialiseResponse * rsp );
extern void ShowSRPresentReqV ( int refnr, SRPresentRequest * req );
extern void ShowSRPresentRespV ( int refnr, SRPresentResponse * rsp );
extern void ShowSRSearchReqV ( int refnr, SRSearchRequest * req );
extern void ShowSRSearchRespV ( int refnr, SRSearchResponse * rsp );
extern void SRdrpComp ( int refnr, SRDeleteResultSetResponse * rsp1, SRDeleteResultSetResponse * rsp2 );
extern void SRdrqComp ( int refnr, SRDeleteResultSetRequest * req1, SRDeleteResultSetRequest * req2 );
extern void SRirpComp ( int refnr, SRInitialiseResponse * rsp1, SRInitialiseResponse * rsp2 );
extern void SRirqComp ( int refnr, SRInitialiseRequest * req1, SRInitialiseRequest * req2 );
extern void SRprpComp ( int refnr, SRPresentResponse * rsp1, SRPresentResponse * rsp2 );
extern void SRprqComp ( int refnr, SRPresentRequest * req1, SRPresentRequest * req2 );
extern void SRsrpComp ( int refnr, SRSearchResponse * rsp1, SRSearchResponse * rsp2 );
extern void SRsrqComp ( int refnr, SRSearchRequest * req1, SRSearchRequest * req2 );
extern void ToLog ( int rmrfnr, char *intxt );
extern void ToLogInp ( int rmrfnr, char *prompt, char *intxt );

